# w.c.s. - web application for online forms
# Copyright (C) 2005-2024  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from wcs.formdef import FormDef

from . import TenantCommand


class Command(TenantCommand):
    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            '--all',
            action='store_true',
            help='wipe all form data',
        )
        parser.add_argument('--forms', metavar='FORMS', help='list of forms (slugs, separated by commas)')
        parser.add_argument(
            '--exclude-forms', metavar='FORMS', help='list of forms to exclude (slugs, separated by commas)'
        )
        parser.add_argument('--no-simulate', action='store_true', help='perform the wipe for real')

    def handle(self, *args, **options):
        if not options.get('no_simulate'):
            self.stdout.write('SIMULATION MODE: no actual wiping will happen.\n')
            self.stdout.write('(use --no-simulate after checking results)\n\n')

        for domain in self.get_domains(**options):
            self.init_tenant_publisher(domain, register_tld_names=False)
            if options.get('all'):
                formdefs = FormDef.select(order_by='url_name')
            elif options.get('forms'):
                formdefs = [FormDef.get_by_urlname(x) for x in options['forms'].split(',')]
            else:
                formdefs = []
            if options.get('exclude_forms'):
                formdefs = [x for x in formdefs if x.url_name not in options['exclude_forms'].split(',')]
            for formdef in formdefs:
                if options.get('no_simulate'):
                    formdef.data_class().wipe()
                else:
                    count = formdef.data_class().count()
                    if count:
                        self.stdout.write(f'{formdef.url_name}: {count}\n')
